# langchain_opentutorial/package.py
import subprocess
import sys
from typing import Optional

import requests

class PackageVersions:
    """
    requirements.txt 기반으로 패키지 버전 정보를 가져옵니다.
    """
    REQUIREMENTS_URL = "https://raw.githubusercontent.com/LangChain-OpenTutorial/LangChain-OpenTutorial/refs/heads/main/requirements.txt"

    @classmethod
    def fetch_versions(cls) -> dict:
        """
        requirements.txt 파일을 가져와 각 패키지와 버전 정보를 딕셔너리로 반환합니다.
        형식: {패키지명: 버전, ...}
        """
        try:
            response = requests.get(cls.REQUIREMENTS_URL)
            response.raise_for_status()
            requirements_text = response.text
            versions = {}
            for line in requirements_text.splitlines():
                line = line.strip()
                # 빈 줄이나 주석은 무시
                if not line or line.startswith("#"):
                    continue
                if "==" in line:
                    package, version = line.split("==", 1)
                    versions[package.strip()] = version.strip()
                else:
                    # 버전 정보가 없는 경우 None 처리 (또는 그대로 패키지명만 저장)
                    versions[line] = None
            return versions
        except requests.RequestException as e:
            print(f"Failed to fetch requirements: {e}")
            return {}

    @classmethod
    def get_version(cls, package: str, release_type_or_date: Optional[str] = None) -> Optional[str]:
        """
        requirements.txt에서 해당 패키지의 버전을 반환합니다.
        release_type_or_date 인자는 이번 구현에서는 무시됩니다.
        """
        versions = cls.fetch_versions()
        return versions.get(package)

def install(packages: list, verbose: bool = True, upgrade: bool = False,
            release_type_or_date: Optional[str] = "stable") -> None:
    """
    requirements.txt에 기반하여 특정 버전의 파이썬 패키지를 설치합니다.

    Args:
        packages (list): 설치할 패키지 이름들의 리스트.
        verbose (bool): 설치 진행 메시지 출력 여부.
        upgrade (bool): 패키지 업그레이드 여부.
        release_type_or_date (str, optional): 기존과 호환을 위해 남겨두었으며, 이번 구현에서는 무시됩니다.
    """
    # 입력 파라미터 검증
    if not isinstance(packages, list):
        raise ValueError("Packages must be provided as a list.")
    if not packages:
        if verbose:
            print("No packages to install.")
        return

    try:
        if verbose:
            print(f"Fetching package versions from: {PackageVersions.REQUIREMENTS_URL}")
            print(f"Installing packages: {', '.join(packages)}...")

        # pip 명령어 준비
        cmd = [sys.executable, "-m", "pip", "install"]
        if upgrade:
            cmd.append("--upgrade")

        # 버전이 명시된 패키지 문자열 생성
        versioned_packages = []
        for package in packages:
            version = PackageVersions.get_version(package, release_type_or_date)
            if version:
                versioned_packages.append(f"{package}=={version}")
            else:
                versioned_packages.append(package)
                if verbose:
                    print(f"Warning: No specific version found for {package}, using latest")

        cmd.extend(versioned_packages)
        subprocess.check_call(cmd, stdout=subprocess.DEVNULL if not verbose else None)

        if verbose:
            print(f"Successfully installed: {', '.join(versioned_packages)}")
    except subprocess.CalledProcessError as e:
        if verbose:
            print(f"Failed to install packages: {', '.join(packages)}")
            print(f"Error: {e}")
